﻿using System;
using System.Collections.Generic;
using System.Text;
using BKSystem.IO;
using System.IO;
using System.Diagnostics;

namespace AVREmulator
{
    public class ScreenBuffer
    {
        public const int CharBits = 8, FrontColorBits = 4, BackColorBits = 4;
        public const int Width = 80, Height = 40, BitsPerChar = CharBits + FrontColorBits + BackColorBits;

        public const int Size = Width * Height * BitsPerChar / 8;

        public readonly byte[] RawMessage;

        public struct Cell {
            public char Character;
            public int FrontColor;
            public int BackColor;
        }
        public readonly Cell[,] Cells = new Cell[Width, Height];

        public ScreenBuffer( byte[] rawMessage )
        {
            Trace.Assert(rawMessage.Length == Size);

            RawMessage = rawMessage;

            BitStream bitStream = new BitStream(new MemoryStream(rawMessage, false));
            for( int row = 0 ; row < Height ; row++ ) {
                for( int column = 0 ; column < Width ; column++ ) {
                    Cell cell;
                    bitStream.Read(out cell.Character, 0, CharBits);
                    bitStream.Read(out cell.FrontColor, 0, FrontColorBits);
                    bitStream.Read(out cell.BackColor, 0, BackColorBits);
                    Cells[column, row] = cell;
                }
            }
        }
    }
}
