#ifndef _APP_H_
#define _APP_H_

/* UIP_APPSTATE_SIZE: The size of the application-specific state
   stored in the uip_conn structure. (not used, but has to at least be one) */
#define UIP_APPSTATE_SIZE 1

#include "stdint.h"
#include "uip.h"

/*
The AVRServer is using a service-based processing scheme for network communication.
A service consists of a service function that gets passed a service request and possibly message data
and returns a service answer, which allows it to take over message handling and control the connection.
*/

typedef enum {
	// the next packet should be passed on to this service (possible fragments)
	SA_MORE_PACKETS,
	// the service wants to keep the connection open to send more data (changed in 2009)
	SA_MORE_PROCESSING = SA_MORE_PACKETS,
	// the service doesnt know what to do with this packet (pass on to the next service)
	SA_PASS_ON,
	// the service has processed this packet (all is well, dont pass on)
	SA_ACCEPT,
	// the service has processed the packet and the connection can be closed now
	SA_FINISH,

	// for answers that only distinguish between FINISH and a different message
	SA_OK
} ServiceAnswer;

//typedef struct {
//	const char *remainingPacket;

typedef enum {
	// process the next packet after a MORE_PACKETS answer
	SR_NEXT_PACKET,
	// probe whether this service can process the packet
	SR_NEW_PACKET,
	SR_RESEND,
	SR_CLOSED,
	SR_ACKED,
	// generic update that is triggered every frame
	SR_POLL
} ServiceRequest;

// because there is too little time to change everything properly, use globals.. :(
/*char *serviceData;
int serviceDataLength;
*/
// I'm only adding this global for now <_<'
// this defaults to the data length, so the new feature doesn't have to be implemented >_>
int serviceDataProcessedLength;

typedef ServiceAnswer (*ServicePacketHandler)( ServiceRequest request, unsigned char *packet, uint16_t length );

// store a packet buffer (with the specified length) for later sending
// i.e. don't change the data that packet points to in the same frame because
// it probably won't have been sent yet
void Service_Send( const unsigned char *packet, uint16_t length );

// returns a pointer to the first byte after the HTTP header
unsigned char * HTTP_skipHeader( const unsigned char *packet, uint16_t length );

// echo ASCII text from an HTTP message (prints with escape sequences)
void HTTP_echoText( const unsigned char *data, uint16_t length );


/*
The following functions can be used to echo messages or parts of them
*/

// TODO: the HTTP namespace is misleading because the functions support raw messages, too
// change to PACKET?

// only echos the header or the beginning of the packet
void HTTP_echoPacketHeader( const unsigned char *packet, uint16_t length );
// only echos the packet part
void HTTP_echoPacketData( const unsigned char *packet, uint16_t length );
// echo the start of the packet (raw data)
void HTTP_echoPacket( const unsigned char *packet, uint16_t length );

// echo a description of the packet (message type & length)
void HTTP_echoPacketInfo( const unsigned char *packet, uint16_t length );




//////
// these functions are used by the main app and are part of the legacy demo app:
//  "A Very Simple Application" from the uIP 0.9 documentation

void server_init(void);
void server_app(void);

#define FS_STATISTICS 0

#define UIP_APPCALL     server_app


#endif
