#ifndef _BUFFER_H
#define _BUFFER_H

#include <avr/io.h>

#include "main.h"
#include "fpga_serial.h"

#ifdef BUFFER_RESET_PACKET
#	include "app.h"
#endif

// constants that are used for sending data packets and processing data from the webclient

// BPS = bits per sample

#define SCREEN_WIDTH 	80
#define SCREEN_HEIGHT 	40
#define SCREEN_BPS		20

#define SCREEN_SIZE  	(SCREEN_WIDTH / 8 * SCREEN_HEIGHT * SCREEN_BPS)

#define FONT_COUNT		256
#define FONT_BPS		(12*8)

#define FONT_SIZE		(FONT_COUNT / 8 * FONT_BPS)

#define COLOR_COUNT		64
#define COLOR_BPS		24

#define COLOR_SIZE		(COLOR_COUNT / 8 * COLOR_BPS);

// init the buffers
void Buffer_Init(void);

// commits any changed buffers as soon as no buffer is acquired anymore
// NOTE: if no buffer is currently acquired, the commit message will be sent straight away
void Buffer_RequestCommit(void);

// acquire a buffer for writing to it (only one buffer can be sent to atm)
void Buffer_Acquire( FPGA_destination destination );

// release the current buffer (flush remaining data if necessary)
void Buffer_Release(void);

// write data to the current buffer (assert that a writing is allowed..)
void Buffer_Write( const unsigned char *data, uint16_t length );

// how many bytes need to be written/can be written
uint16_t Buffer_GetRemaining(void);

// TODO: remove this function if it isnt used at all..
// total buffer size (of the current buffer)
//uint16_t Buffer_GetSize(void);

// remaining bytes == 0?
bool Buffer_Done(void);

// is a buffer currently acquired?
bool Buffer_Acquired(void);

// this function is used by the webclient to write formatted text to the buffer (and wrap automatically)
// screen color: front color << 4 + back color
void Buffer_WriteEncodedScreenData( unsigned char frontColor, unsigned char backColor, unsigned char *data, uint16_t length );

#ifdef BUFFER_RESET_PACKET
ServiceAnswer Buffer_handleResetPacket( ServiceRequest request, unsigned char *packet, uint16_t length );
#endif

#endif
