#include <avr/io.h>
#include <util/delay.h>

#include "config.h"
#include "fpga_serial.h"

#ifdef FPGA_DEBUG_COMMAND_BYTES
#	include "serial.h"
#endif

#ifdef FPGA_LED_DEBUG
#define DELAY() 	do {\
						unsigned m;\
						for (m=0;m<10;m++)\
							_delay_ms(10);\
        			} while( 0 )
#else
#define DELAY()		
#endif

// output a bit on the data line and output a clock pulse too to let the FPGA know about it, too
#define FPGA_DATA_OUT( state ) 	do {\
									if( state )\
										FPGA_PORT |= FPGA_DATA_MASK;\
									else\
										FPGA_PORT &= ~FPGA_DATA_MASK;\
									FPGA_PORT |= FPGA_CLK_MASK;\
									DELAY();\
									FPGA_PORT &= ~FPGA_CLK_MASK;\
									DELAY();\
								} while( 0 )

#define FPGA_BEGIN()			do { FPGA_PORT |= FPGA_MWE_MASK; } while( 0 )
// send an additional clock to make the FPGA team happy
// the happy chaps from the FPGA team told me that the additional clock isnt needed anymore, so here it goes..
#define FPGA_END()				do { FPGA_PORT &= ~(FPGA_MWE_MASK | FPGA_DATA_MASK); /*FPGA_PORT |= FPGA_CLK_MASK; FPGA_PORT &= ~FPGA_CLK_MASK;*/ } while( 0 )

// this is used to build the command | buffer mask prefix (the first byte that is sent to the FPGA)
#define FPGA_PREFIX( c, m )		((c << 4) + (m & 0xF))

static inline void FPGA_pushCommand( unsigned char byte );
static inline void FPGA_pushData( unsigned char data );
static inline void FPGA_push12BitData( unsigned short data );

static unsigned short FPGA_currentColor;

static inline void FPGA_pushData( unsigned char data ) {
	unsigned i;
	
	for( i = 0 ; i < 8 ; i++, data <<= 1 ) {
		FPGA_DATA_OUT( data & (1<<7) );
	}
}

static inline void FPGA_push12BitData( unsigned short data ) {
	unsigned i;
	
	for( i = 0 ; i < 12 ; i++, data <<= 1 ) {
		FPGA_DATA_OUT( data & (1<<11) );
	}
}

// set the color state
void FPGA_setColor( unsigned char frontColor, unsigned char backColor ) {
	//FPGA_currentColor = (frontColor & 0x0F) + ((backColor & 0x0F) << 4);
	const char mask = (1 << 6) - 1;
	FPGA_currentColor = (backColor & mask) + ((frontColor & mask) << 6);
}

// color + char
void FPGA_pushChar( unsigned char character ) {
	FPGA_pushData( character );
	//FPGA_pushData( FPGA_currentColor );
	FPGA_push12BitData( FPGA_currentColor );
}

void FPGA_beginData( FPGA_destination destination ) {
	FPGA_BEGIN();
	
	FPGA_pushCommand( FPGA_PREFIX( FC_DATA, FD_TO_FM( destination ) ) );
}

void FPGA_endData(void) {
	FPGA_END();
}

void FPGA_sendData( FPGA_destination destination, const unsigned char *data, unsigned length ) {
	FPGA_beginData( destination );
	
	for( ; length > 0 ; length-- ) {
		FPGA_pushData( *data++ );
	}
	
	FPGA_endData();
}

void FPGA_flushData( FPGA_destination destination, int32_t length ) {
	FPGA_beginData( destination );
	
	for( ; length > 0 ; --length ) {
		FPGA_DATA_OUT( 0 );
	}
		
	FPGA_endData();
}

static void FPGA_pushCommand( unsigned char byte ) {
	unsigned i;

#ifdef FPGA_DEBUG_COMMAND_BYTES
	xputs( "Command Bits: " );
#endif
	
	for( i = 0 ; i < 8 ; i++, byte <<= 1 ) {
		FPGA_DATA_OUT( byte & 0x80 );

#ifdef FPGA_DEBUG_COMMAND_BYTES
		xputc( (byte & 0x80) ? '1' : '0' );
#endif
	}

#ifdef FPGA_DEBUG_COMMAND_BYTES
		xputs( "\r\n" );
#endif
}

void FPGA_sendCommand( FPGA_command command, uint8_t destinationMask /*use ORed FPGA_mask */ ) {
	FPGA_BEGIN();

	FPGA_pushCommand( FPGA_PREFIX( command, destinationMask ) );

	FPGA_END();
}

void FPGA_init(void) {
	FPGA_DIRECTION |= FPGA_MWE_MASK | FPGA_DATA_MASK | FPGA_CLK_MASK;
}
