#ifndef _FPGA_SERIAL_H
#define _FPGA_SERIAL_H

/* low-level FPGA memory interface

This is a primitive interface to the FPGA. It serializes data according to the communication protocol.
It doesn't contain any higher logical. Other systems handle this (ie. Buffer for resetting the FPGA state).
*/

// configuration of the serializer
#define FPGA_PORT 		PORTB
#define FPGA_DIRECTION	DDRB
#define FPGA_MWE_MASK 	(1 << 0)
#define FPGA_CLK_MASK 	(1 << 1)
#define FPGA_DATA_MASK 	(1 << 2)

// specify the address and data that should be written into the FPGA's memory in sequence
// NOTE: this all must stay synchronized between AVR and FPGA!
typedef enum {
	FD_FONT,
	FD_SCREEN,
	FD_COLOR,
	FD_COUNT
} FPGA_destination;

// destination to mask
#define FD_TO_FM( x )	(1 << (x))

typedef enum {
	FM_FONT = 1 << FD_FONT,
	FM_SCREEN = 1 << FD_SCREEN,
	FM_COLOR = 1 << FD_COLOR
} FPGA_mask;

typedef enum {
	// data block following
	FC_DATA = 0x0,
	// commit the buffer masks to the front buffer
	FC_COMMIT = 0x1,
	// reset the buffer indices/state
	FC_RESET = 0xF
} FPGA_command;

// send data to the fpga
void FPGA_sendData( FPGA_destination destination, const unsigned char *data, unsigned length );
// flush the fpga with length bits (on a specific destination buffer)
void FPGA_flushData( FPGA_destination destination, int32_t length );

// use these function if you need to push bytes one by one
// first call beginData, then push repeatedly to send the data and finally endData to finish the transmission
void FPGA_beginData( FPGA_destination destination );
void FPGA_endData(void);
// TODO: clean this up on the FPGA side someday?


// set the color state
void FPGA_setColor( unsigned char frontColor, unsigned char backColor );
// sends the current color along with the character
void FPGA_pushChar( unsigned char character );

// send a control command to the FPGA
// it is recommened that you do not send FC_DATA because it doesn't really make sense..
void FPGA_sendCommand( FPGA_command command, uint8_t destinationMask /*use ORed FPGA_mask */ );

// init the FPGA serializer
void FPGA_init(void);

#endif
