#include <stdlib.h>
#include <avr/pgmspace.h>
#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "config.h"

// TODO: move the destination stuff to another enum
#include "logservice.h"

#define LOGBUFFER_SIZE 			(2<<10)

static char *logBuffer = NULL;
static int logBufferLength = 0;

//static uint16_t sentBytes[ UIP_CONNS ];
//static uint16_t ackedBytes[ UIP_CONNS ];

#define NO_CONNECTION -1
static int connectionIndex = NO_CONNECTION;

#define LogService_GET_CMD		"GET /log"

static const char httpMessageHeader[] =
	"HTTP/1.0 200 OK\r\n"
	"Content-Type: text/plain\r\n"
	"\r\n\r\n";

void LogService_init(void) {
	logBuffer = malloc(LOGBUFFER_SIZE + 1);
	logBuffer[0] = 0;
	logBufferLength = 0;
}

void LogService_add(const char *msg, int length) {
	// disabled backlogging because it doesnt look that nice (the log service messages get cut off as soon as it understands that its a log connection..)
	if( connectionIndex == uip_getcurindex() || connectionIndex == NO_CONNECTION ) {
		return;
	}
	//xputc('.');

	if( length > LOGBUFFER_SIZE / 2 ) {
		length = LOGBUFFER_SIZE / 2;
	}
	// move buffer around?
	int overflowLength = logBufferLength + length - LOGBUFFER_SIZE;
	if( overflowLength > 0 ) {
		// remove half of the log
		if( logBufferLength > LOGBUFFER_SIZE / 2) {
			logBufferLength = LOGBUFFER_SIZE / 2;
			memmove( logBuffer, logBuffer + LOGBUFFER_SIZE / 2, logBufferLength);
		}		
	}

	// add the null character, too
	memcpy( logBuffer + logBufferLength, msg, length + 1 );
	logBufferLength += length;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////
// Service to send the log (as test file) to a client (using multiple packets if necessary)

// returns FALSE if all has been sent
static ServiceAnswer LogService_sendPacket(void) {
	uint8_t currentIndex = uip_getcurindex();

	//	sprintf( pTextBuffer, "mss: %i acked: %i sent: %i length: %i\r\n", uip_mss(), ackedBytes[ currentIndex ], sentBytes[ currentIndex ], length );
	//	xputs( pTextBuffer );

	if( logBufferLength > 0 ) {
		Service_Send( logBuffer, logBufferLength );
	}
	logBufferLength = 0;
	return TRUE;
}

static void LogService_startSending(void) {
	connectionIndex = uip_getcurindex();

	Service_Send( httpMessageHeader, sizeof(httpMessageHeader) - 1 /* dont send the null char */);
}

ServiceAnswer LogService_handleGetPacket( ServiceRequest request, unsigned char *data, uint16_t length ) {
	if( request == SR_NEW_PACKET ) {
		if( strncmp( data, LogService_GET_CMD, sizeof( LogService_GET_CMD ) - 1 ) == 0 ) {
			if( connectionIndex == NO_CONNECTION ) {
				LogService_startSending();
				return SA_MORE_PROCESSING;
			}
			else {
				// close the connection (only one allowed atm)
				return SA_FINISH;
			}
		}
	}
	else if( request == SR_POLL ) {
		if( LogService_sendPacket() ) {
			return SA_OK;
		} 
	}
	else if( request == SR_CLOSED ) {
		connectionIndex = NO_CONNECTION;
		return SA_OK;
	}
	
	return SA_PASS_ON;
}


