#include <string.h>
#include <stdlib.h>
#include <stdlib.h>

#include "uipopt.h"

#include "config.h"
#include "main.h"
#include "packetbuffer.h"
#include "serial.h"

// alternative implementation without original data below..
#ifdef FAST_PACKETBUFFER
// we can be a bit more wasteful now
#define packetBufferSize  (UIP_TCP_MSS + MIN_RAW_PACKET_SIZE)
static char (*packetBuffer)[packetBufferSize] = NULL;
// 0 signals that the packet buffer is empty
static int packetBufferLength[UIP_CONNS];

// NULL signals that the original data feature is not used
static char *originalData[UIP_CONNS];
static int originalDataLength[UIP_CONNS];

char * PacketBuffer_getData(void) {
	if( originalData[uip_getcurindex()] ) {
		return originalData[uip_getcurindex()];
	}
	return packetBuffer[uip_getcurindex()];
}

uint16_t PacketBuffer_getLength(void) {
	if( originalData[uip_getcurindex()] ) {
		return originalDataLength[uip_getcurindex()];
	}
	return packetBufferLength[uip_getcurindex()];
}

void PacketBuffer_pushData( char *data, uint16_t length ) {
	if( length == 0 ) {
		return;
	}

	// if we have original data, copy it over
	PacketBuffer_makePersistent();

	// is the buffer empty -> use original data
	if( packetBufferLength[uip_getcurindex()] == 0 ) {
		originalData[uip_getcurindex()] = data;
		originalDataLength[uip_getcurindex()] = length;
	}
	else {
		// append the new data
		if( length + packetBufferLength[uip_getcurindex()] >= packetBufferSize ) {
			xprintf( "ERROR: cant add data to the packet buffer (%i + %i >= %i)!\r\n", length, packetBufferLength[uip_getcurindex()], packetBufferSize );
			MSG("--ERR: overflow in packet buffer prevented" );
			// reset the packet buffer
			// we dont want to lock the buffer forever >_>
			packetBufferLength[uip_getcurindex()] = 0;
			return;
		}
		memcpy( packetBuffer[uip_getcurindex()] + packetBufferLength[uip_getcurindex()], data, length );
		packetBufferLength[uip_getcurindex()] += length;
	}
}

void PacketBuffer_popData( uint16_t length ) {
	// original data?
	if( originalData[uip_getcurindex()] ) {
		char *orgData = originalData[uip_getcurindex()];
		originalData[uip_getcurindex()] = NULL;
		packetBufferLength[uip_getcurindex()] = 0;

		if( length < originalDataLength[uip_getcurindex()] ) {
			// copy the remaining data into the normal buffer
		    packetBufferLength[uip_getcurindex()] = originalDataLength[uip_getcurindex()] - length;
			memcpy( packetBuffer[uip_getcurindex()], orgData + length, packetBufferLength[uip_getcurindex()] );
		}
	}
	else {
		// packet buffer
		if( length >= packetBufferLength[uip_getcurindex()] ) {
			packetBufferLength[uip_getcurindex()] = 0;
		}
		else {
			packetBufferLength[uip_getcurindex()] -= length;
			memmove( packetBuffer[uip_getcurindex()], packetBuffer[uip_getcurindex()] + length, packetBufferLength[uip_getcurindex()] );
		}
	}
}

void PacketBuffer_init(void) {
	packetBuffer = malloc( packetBufferSize );
}

void PacketBuffer_initConnection(void) {
	packetBufferLength[uip_getcurindex()] = 0;
	originalData[uip_getcurindex()] = NULL;
	originalDataLength[uip_getcurindex()] = 0;
}

void PacketBuffer_makePersistent(void) {
	if( originalData[uip_getcurindex()] ) {
		// assert: packetBufferLength == 0
		memcpy( packetBuffer[uip_getcurindex()], originalData[uip_getcurindex()], originalDataLength[uip_getcurindex()] );

		originalData[uip_getcurindex()] = NULL;
		packetBufferLength[uip_getcurindex()] = originalDataLength[uip_getcurindex()];
	}
}
#else

// we can be a bit more wasteful now
static const int packetBufferSize = UIP_TCP_MSS + MIN_RAW_PACKET_SIZE;
static char *packetBuffer = NULL;
// 0 signals that the packet buffer is empty
static int packetBufferLength = 0;

char * PacketBuffer_getData(void) {
	return packetBuffer;
}

uint16_t PacketBuffer_getLength(void) {
	return packetBufferLength;
}

void PacketBuffer_pushData( char *data, uint16_t length ) {
	if( length == 0 ) {
		return;
	}

	// append the new data
	if( length + packetBufferLength >= packetBufferSize ) {
		xprintf( "ERROR: cant add data to the packet buffer (%i + %i >= %i)!\r\n", length, packetBufferLength, packetBufferSize );
		MSG("--ERR: overflow in packet buffer prevented" );
		// reset the packet buffer
		// we dont want to lock the buffer forever >_>
		packetBufferLength = 0;
		return;
	}
	memcpy( packetBuffer + packetBufferLength, data, length );
	packetBufferLength += length;
}

void PacketBuffer_popData( uint16_t length ) {
	// packet buffer
	if( length >= packetBufferLength ) {
		packetBufferLength = 0;
	}
	else {
		packetBufferLength -= length;
		// dont use memmove to make sure the error wont appear because of it
		//memmove( packetBuffer, packetBuffer + length, packetBufferLength );
		char *temp = malloc(packetBufferLength);
		memcpy(temp, packetBuffer + length, packetBufferLength);
		memcpy(packetBuffer, temp, packetBufferLength);
		free(temp);
	}
}

void PacketBuffer_init(void) {
	packetBuffer = malloc( packetBufferSize * UIP_CONNS );
	packetBufferLength = 0;
}

void PacketBuffer_makePersistent(void) {
}
#endif
