@echo off
setlocal ENABLEEXTENSIONS ENABLEDELAYEDEXPANSION
set avr_ip="131.159.35.20"
set avr_port=80

echo AVR Testsuite
call :print_ip

:cmdparser
	if /i "%1" == "--ip" goto :setip
	if /i "%1" == "--test" goto :testsend
	if /i "%1" == "--raw" goto :rawsend
	if /i "%1" == "--web" goto :websend
	if /i "%1" == "--help" goto :help
	if /i "%1" == "--reset" goto :sendreset
	if not "%1" == "" (
		echo Unknown Command: %1 - try --help for a short description
		shift
	)

endlocal
goto :eof

REM cmd parsing
:setip 
	shift
	set avr_ip=%~1
	shift

	call :print_ip

	goto :cmdparser

:testsend
	shift
	
	setlocal
	set buffer=
	set data=
	set delay=
	set swap=noswap
	
	call :parse_buffer %1
	shift

	if /i "%1"=="swap" (
		set swap=swap
		shift
	)
	
	REM send the message
	echo Sending Message...
	REM cat %buffer%.header %swap%.suffix %data%.data 
	call :send_raw_message "%buffer%.header" "%swap%.suffix" "%data%.data"
	
	endlocal
	goto :cmdparser
	
:rawsend
	shift
	
	setlocal
	set buffer=
	set data=
	set delay=
	set swap=noswap
		
	call :parse_buffer %1
	shift

	set file=%~1
	shift
	
	if /i "%1"=="swap" (
		set swap=swap
		shift
	)
	
	REM send the message
	call :send_raw_message "%buffer%.header" "%swap%.suffix" "%file%"
	
	endlocal
	goto :cmdparser
	
:websend
	shift
	
	setlocal
	
	REM foreground
	call :convert_to_hex %1
	shift
	
	set foreground=%return%
	
	REM background
	call :convert_to_hex %2
	shift
	
	set background=%return%
	
	endlocal
	goto :cmdparser
	
:sendreset
	shift
	echo Sending a reset packet
	nc %avr_ip% %avr_port% -w 2 < reset.msg
	
	goto :cmdparser
	
:help
	shift 
	echo.
	echo AVR Testsuite help:
	echo.
	echo testsuite [commands]*
	echo.
	echo Possible Commands:
	echo	--help
	echo		displays this help
	echo.
	echo	--ip [ip-address]
	echo		changes the AVR server ip
	echo.
	echo	--test [buffer-type] {[swap]}
	echo		sends a test message to the AVR (using default data). use swap to swap all changed buffers on the AVR
	echo.
	echo		possible values for buffer-type: 	screen, color, font
	echo	--raw [buffer-type] [filename] {[swap]} 
	echo		sends a test message to the AVR (using default data). use swap to swap all changed buffers on the AVR
	echo.
	echo		possible values for buffer-type: 	screen, color, font
	echo.	
	echo	--reset
	echo		sends a reset packet to the AVR (only works if the respective AVR build supports the feature)
	echo.
	goto :eof

REM helper scripts
:print_ip
	echo Using AVR Server IP %avr_ip%
	goto :eof

:parse_buffer
	if /i "%1"=="screen" (
		set data=8000
		set buffer=screen
		set delay=30
	) else if /i "%1"=="font" (
		set data=3072
		set buffer=font
		set delay=12
	) else if /i "%1"=="color" (
		set data=192
		set buffer=color
		set delay=3
	) else (
		echo Unknown buffer %1
	)
	
	goto :eof
	
:send_raw_message
REM header param data

	echo Building Message...
	REM copy /B %1 + /B %2 + /B %3 message.tmp /Y /B > nul
	copy /B %1 + /B %2 + /B %3 message.tmp /Y /B
	echo Sending Message (Delay: %delay% s)
	nc %avr_ip% %avr_port% -w %delay% < message.tmp
	del message.tmp
	echo Message Sent.
	
	goto :eof
	
:convert_to_hex:
REM value -> return
	set return=
	set /a value=%1+1-1
	
	REM range check
	if %value% LEQ 0 (
		set return=00
		goto :eof
	) else if %value% GEQ 64 (
		set return=00
		goto :eof
	)
	
	REM setup the prefix to make it 2 digit
	set prefix=
	if %value% LSS 16 (
		set prefix=0
	)
	
	:digit_loop
		set /a digit="value & 0xF"
		
		if %value% EQU 0 (
			set return=%prefix%%return%
						
			goto :eof
		)
				
		if %digit% LSS 0xA (
			set return=%digit%%return%
		) else if %digit% EQU 0xA (
			set return=A%return%
		) else if %digit% EQU 0xB (
			set return=B%return%
		) else if %digit% EQU 0xC (
			set return=C%return%
		) else if %digit% EQU 0xD (
			set return=D%return%
		) else if %digit% EQU 0xE (
			set return=E%return%
		) else if %digit% EQU 0xF (
			set return=F%return%
		)
		
		set /a value=value / 16
		
		goto :digit_loop
		
goto :eof
